var propertiesResponse = null;

$(document).ready(function() {

  initTooltips();
  initVideoChannel();
  initServiceId();

  // Fill form
  Switch().init('pipEnabled');

  $('#pipCustomName')
    .val(serviceArrTmp[activeItem].customName || '')
    .keyup(onCustomNameKeyUp)
    .attachValidation({
      pattern: '^[A-Za-z0-9 -]{0,}$',
      invalidMsg: 'Only chars A-Z, a-z, 0-9, white space and dash are allowed.',
      disableBtnId: 'saveBtn'
    });

  $('#pipPosX').val(serviceArrTmp[activeItem].pos_x);
  $('#pipPosY').val(serviceArrTmp[activeItem].pos_y);

  var fps = typeof serviceArrTmp[activeItem].fps == 'number' ? serviceArrTmp[activeItem].fps : 1;
  $('#pipFPS').val(fps.toFixed(fps < 1 ? 1 : 0));
  $('#pipCompression').val(serviceArrTmp[activeItem].compression || '50');

  initPipSourceType();

  const resolutionId = '#pipResolution';
  const videoChannelId = '#pipVideoChannel';
  createResolutionList(resolutionId);
  createPositionList('#pipPosition', serviceArrTmp[activeItem].coordSystem);
  createTriggeringElement('#pipAutomationType', serviceArrTmp[activeItem].automationType, serviceArrTmp[activeItem].schedule);
  onVideoChannelChange(resolutionId, videoChannelId, serviceArrTmp[activeItem].camera);
  createVideoChannelElement(videoChannelId, serviceArrTmp[activeItem].camera).then(function(){
    // Callbacks for changes
    $('.form-control').change(formChangedCallback);

    $('#checkCameraConnection').on('click', function() {
      getProxyCameraData(100)
        .then(function () {

          showConnectionState(true, "Connection successful");
      }).catch(function(err) {

          showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
          console.log("Cannot connect to camera. Check the details of the remote camera.", err);
      });
    });

    if (serviceArrTmp[activeItem].camera_ip.length && serviceArrTmp[activeItem].camera_user.length && serviceArrTmp[activeItem].camera_pass.length) {
      getProxyCameraData(4000)
        .catch(function(err) {

          showConnectionState(false, "Cannot connect to camera. Check the details of the remote camera.");
          console.log("Cannot connect to camera. Check the details of the remote camera.", err);
      });
    } else {
      $("#cameraInfoForm").hide();
    }

    formChangedCallback();
  });

  createScheduleBtn();
});

function initPipSourceType() {
  isActiveMjpeg = (serviceArrTmp[activeItem].source_type === "mjpeg_url");

  //Fill data
  $('#pipMjpegUrl').val(serviceArrTmp[activeItem].mjpeg_url);
  $('#pipMjpegWidth').val(serviceArrTmp[activeItem].mjpeg_width);
  $('#pipMjpegHeight').val(serviceArrTmp[activeItem].mjpeg_height);
  $('#pipCameraWidth').val(serviceArrTmp[activeItem].camera_width);
  $('#pipCameraHeight').val(serviceArrTmp[activeItem].camera_height);
  $('#pipCameraIP').val(serviceArrTmp[activeItem].camera_ip);
  $('#pipCameraPort').val(serviceArrTmp[activeItem].camera_port);
  $('#pipCameraUser').val(serviceArrTmp[activeItem].camera_user);
  $('#pipCameraPass').val(serviceArrTmp[activeItem].camera_pass);

  //Active tab
  var tabClass = isActiveMjpeg ? '.mjpeg_url' : '.axis_camera';
  $('#pipSourceType .nav-link'+tabClass).tab('show');

  //Disable or enable FPS and Compression on MJPEG
  toggleFpsAndCompression(isActiveMjpeg);

  //Register event on tab click
  $('#pipSourceType .nav-link.axis_camera').off('click').on('click', function() {
    serviceArrTmp[activeItem].source_type = "axis_camera";
    toggleFpsAndCompression(false);
  });

  $('#pipSourceType .nav-link.mjpeg_url').off('click').on('click', function() {
    serviceArrTmp[activeItem].source_type = "mjpeg_url";
    toggleFpsAndCompression(true);
  });

}

function toggleFpsAndCompression(disabled) {
  $('#pipFPS').attr('disabled', disabled);
  $('#pipCompression').attr('disabled', disabled);
}

function formChangedCallback() {
  serviceArrTmp[activeItem].customName = $('#pipCustomName').val();



  let currentCameraResolutionObject = cameraResolutions[parseInt($('#pipResolution').val())];
  serviceArrTmp[activeItem].width = currentCameraResolutionObject.width;
  serviceArrTmp[activeItem].height = currentCameraResolutionObject.height;
  serviceArrTmp[activeItem].coordSystem = $('#pipPosition').val();
  serviceArrTmp[activeItem].automationType = $('#pipAutomationType').val();
  serviceArrTmp[activeItem].pos_x = parseInt($('#pipPosX').val());
  serviceArrTmp[activeItem].pos_y = parseInt($('#pipPosY').val());
  serviceArrTmp[activeItem].fps = parseFloat($('#pipFPS').val());
  serviceArrTmp[activeItem].compression = parseInt($('#pipCompression').val());

  serviceArrTmp[activeItem].mjpeg_url = $('#pipMjpegUrl').val().trim();
  serviceArrTmp[activeItem].mjpeg_width = parseInt($('#pipMjpegWidth').val());
  serviceArrTmp[activeItem].mjpeg_height = parseInt($('#pipMjpegHeight').val());

  serviceArrTmp[activeItem].camera_ip = $('#pipCameraIP').val();
  serviceArrTmp[activeItem].camera_port = parseInt($('#pipCameraPort').val());
  serviceArrTmp[activeItem].camera_user = $('#pipCameraUser').val();
  serviceArrTmp[activeItem].camera_pass = $('#pipCameraPass').val();

  if (propertiesResponse && $('#pipCameraViewArea').val() != null) {

    if (serviceArrTmp[activeItem].camera_view_area != $('#pipCameraViewArea').val()) {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
      createCameraResolutionList(serviceArrTmp[activeItem].camera_view_area - 1, propertiesResponse);
    } else {
      serviceArrTmp[activeItem].camera_view_area = $('#pipCameraViewArea').val();
    }

    var cmResItem = $('#pipCameraResolution')[0];
    if(cmResItem.length > 0) {
      var res = cmResItem.options[cmResItem.selectedIndex].text;
      res = res.split('x');
      serviceArrTmp[activeItem].camera_width = parseInt(res[0].trim());
      serviceArrTmp[activeItem].camera_height = parseInt(res[1].trim());
    }
  }

  serviceArrTmp[activeItem].camera = saveVideoChannelValue(serviceArrTmp[activeItem].camera, $('#pipVideoChannel'));
  createCameraPreview($('#pipVideoChannel'), currentCameraResolutionObject, serviceArrTmp[activeItem].camera);

  showSaveWarning();
  createServiceList(serviceArrTmp);
}

function waitForTimeout(timeout) {
  return new Promise(function (resolve) {
    setTimeout(resolve, timeout);
  });
}

function getProxyCameraData(time) {

  $('#cameraInfoForm').showSpinner('Loading data');

  return waitForTimeout(time)
    .then(function () {
      return getProxyCameraParam('Image.NbrOfConfigs');
    })
    .then(function(configResponse) {
      if (!configResponse.hasOwnProperty('image.nbrofconfigs')) {
        throw new Error('Connection error 1 - can\'t get params of proxy camera');
      }
      var configCount = parseInt(configResponse['image.nbrofconfigs']);
      var params = 'Properties.Image.Resolution,image.nbrofconfigs';
      for (var i = 0; i < configCount; i++) {
        params += ',Properties.Image.I' + i + '.Resolution,Image.I' + i + '.Name,Image.I' + i + '.Enabled';
      }
      return params;

    }).then(function(params) {
      return getProxyCameraParam(params);

    }).then(function(propsResponse) {
        if (!propsResponse.hasOwnProperty('properties.image.resolution')) {
          throw new Error('Connection error 2 - can\'t get params of proxy camera');
        }
        propertiesResponse = propsResponse;
        var configCount = parseInt(propsResponse['image.nbrofconfigs']);
        var selectedViewArea = createCameraViewAreaList(configCount, propertiesResponse); // Return selected view area number
        createCameraResolutionList(selectedViewArea, propertiesResponse);

    }).catch(function(err) {
      console.log(err);
      throw new Error("Connection error 4 - can't connect to proxy camera.");

    }).finally(function () {

      $('#cameraInfoForm').hideSpinner();
    });
}

function getProxyCameraParam(paramList) {
  return new Promise(function(resolve, reject) {
    let url = new URL('/local/camoverlay/api/proxy.cgi', window.location.origin);
    let urlParams = new URLSearchParams(url.search);
    urlParams.set('target_camera_ip', serviceArrTmp[activeItem].camera_ip);
    urlParams.set('target_camera_port', serviceArrTmp[activeItem].camera_port);
    urlParams.set('target_camera_path', '/axis-cgi/param.cgi');
    urlParams.set('target_camera_user', serviceArrTmp[activeItem].camera_user);
    urlParams.set('target_camera_pass', serviceArrTmp[activeItem].camera_pass);
    urlParams.set('action', 'list');
    urlParams.set('group', paramList);

    url.search = urlParams.toString();

    $.get(url, function(response) {
      resolve(parseParams(response));
    }).fail(function() {
      reject(new Error('Network error.'));
    });
  });
}

function parseResolutions(resString) {
  var resolutions = [];
  var resArr = resString.split(',');
  for (var i = 0; i < resArr.length; i++) {
    var resItems = resArr[i].split('x');
    if (resItems.length > 1) {
      resolutions.push({"width": + parseInt(resItems[0]), "height": + parseInt(resItems[1])});
    }
  }
  resolutions = resolutions.sort(function(a, b) {
    return b.width - a.width;
  });
  return resolutions;
}

function createCameraViewAreaList(viewAreaCount, cameraProperties) {
  var selected = -1;
  $('#pipCameraViewArea').empty();
  var resItem = $('#pipCameraViewArea')[0];
  for (var i = 0; i < viewAreaCount; i++) {
    if (cameraProperties['image.i' + i + '.enabled'] == 'yes') {
      if (i + 1 == serviceArrTmp[activeItem].camera_view_area) {
        selected = i;
      }
      resItem.options[resItem.options.length] = new Option(cameraProperties['image.i' + i + '.name'], i + 1);
    }
  }
  if (selected == -1) {
    selected = 0;
    if(resItem.options.length === 0){ // missing view areas in PTZ camera, add 'Default' value
      resItem.options[0] = new Option('Default', 1);
    }
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function createCameraResolutionList(viewArea, cameraProperties) {
  var resolutions = cameraProperties['properties.image.resolution'];
  if (cameraProperties.hasOwnProperty('properties.image.i' + viewArea + '.resolution')) {
    resolutions = cameraProperties['properties.image.i' + viewArea + '.resolution'];
  }
  resolutions = parseResolutions(resolutions);

  var selected = -1;
  $('#pipCameraResolution').empty();
  var resItem = $('#pipCameraResolution')[0];
  for (var i = 0; i < resolutions.length; i++) {
    if (resolutions[i].width == serviceArrTmp[activeItem].camera_width && resolutions[i].height == serviceArrTmp[activeItem].camera_height) {
      selected = i;
    }
    resItem.options[resItem.options.length] = new Option(resolutions[i].width + ' x ' + resolutions[i].height, i);
  }
  if (selected == -1) {
    selected = 0;
    formChangedCallback();
  }
  resItem.selectedIndex = selected;
  return selected;
}

function showConnectionState(state, text) {
  if (text !== undefined && text !== '') {
    if (state) {
      showMessage('success', text, false, 'pipConnectionStateSuccess');
      $("#cameraInfoForm").show();
    } else {
      showMessage('danger', text, false, 'pipConnectionStateError');
      $("#cameraInfoForm").hide();
    }
  }
}