#!/bin/sh
# Enable CGIs on Apache web server

restartHttpd()
{
  if [ `pidof systemd` -eq 1 ]; then
    systemctl reload httpd
  else
    /usr/sbin/httpd -k graceful
  fi
  return `echo $?`
}

APP_NAME=camoverlay
APP_ROOT_DIR=/usr/local/packages/${APP_NAME}
eval $(grep VERSION $APP_ROOT_DIR/package.conf)

HTTPD_VHOST_CONF=/etc/apache2/conf.d/vhosts/all/custom_${APP_NAME}.conf
HTTPD_VHOST_CONF_TMP=${HTTPD_VHOST_CONF}.tmp

HTTPD_VHOST_BASIC=/etc/apache2/conf.d/vhosts/localhost-basic/custom_${APP_NAME}.conf
HTTPD_VHOST_DIGEST=/etc/apache2/conf.d/vhosts/localhost-digest/custom_${APP_NAME}.conf
HTTPD_VHOST_DEFAULT=/etc/apache2/conf.d/vhosts/http/custom_${APP_NAME}.conf
HTTPD_VHOST_DEFAULT_HTTPS=/etc/apache2/conf.d/vhosts/https/custom_${APP_NAME}.conf

if [ -e /etc/apache2/httpd.conf ]; then

# Don't change content of this file!!! There is no detection to restart HTTPd service.
  cat > $HTTPD_VHOST_BASIC << EOF
<Proxy "*">
    Include /etc/apache2/httpd-basic.conf
    Require axis-group admin
    RequestHeader unset Authorization
</Proxy>
EOF
# Don't change content of this file!!! There is no detection to restart HTTPd service.
  cat > $HTTPD_VHOST_DIGEST << EOF
<Proxy "*">
    Include /etc/apache2/httpd-digest.conf
    Require axis-group admin
    RequestHeader unset Authorization
</Proxy>
EOF
# Don't change content of this file!!! There is no detection to restart HTTPd service.
  cat > $HTTPD_VHOST_DEFAULT << EOF
<Proxy "*">
    Include /run/apache2/httpd-select-auth.conf
    Require axis-group admin
    RequestHeader unset Authorization
</Proxy>
EOF
   cp $HTTPD_VHOST_DEFAULT $HTTPD_VHOST_DEFAULT_HTTPS

  cat > $HTTPD_VHOST_CONF_TMP << EOF
LoadModule proxy_http_module lib/apache2/modules/mod_proxy_http.so
LoadModule proxy_wstunnel_module lib/apache2/modules/mod_proxy_wstunnel.so
Alias "/local/${APP_NAME}/$APPMAJORVERSION-$APPMINORVERSION-$APPMICROVERSION" "$APP_ROOT_DIR/html"
<Directory "$APP_ROOT_DIR/html">
    Require all granted
</Directory>
<LocationMatch "^/local/${APP_NAME}/api/.*\.cgi$">
    Require axis-group admin
    ProxyPass "http://127.0.0.1:52430/"
</LocationMatch>
<Location "/local/${APP_NAME}/ws">
    Require axis-group admin
    ProxyPass "ws://127.0.0.1:52431/"
</Location>
EOF

  # Compare configuration files and reload httpd only if there is a change
  cmp $HTTPD_VHOST_CONF $HTTPD_VHOST_CONF_TMP
  VHOST_CMP=$?
  if [ $VHOST_CMP -eq 0 ]; then
    rm -f $HTTPD_VHOST_CONF_TMP
  else
    mv $HTTPD_VHOST_CONF_TMP $HTTPD_VHOST_CONF
    chmod 600 $HTTPD_VHOST_CONF
    chown www:www  $HTTPD_VHOST_CONF

    restartHttpd
    if [ $? -ne 0 ]; then
      logger -t "CamOverlay" "Incompatible firmware: Upgrade the firmware version."
      rm -f $HTTPD_VHOST_CONF $HTTPD_VHOST_BASIC $HTTPD_VHOST_DIGEST $HTTPD_VHOST_DEFAULT $HTTPD_VHOST_DEFAULT_HTTPS
      restartHttpd
      exit 1
    fi
  fi
fi
